"""
pacController.py

Cool things does in this code:
1) Creation of KeyPad bindings for moving a PacMan around...
2) Grabbing a node (by name) when your sure the model has only one such node
3) Modifying the value of a node and updating it's visual appearence
4) Deletion of a link between two nodes
5) Creation of a link between two nodes
6) A hack to find if a visual object exists in the near vincinity of another,
this is only valid for use in cases where very few objects exist! Otherwise,
use of named ports and manual constraint checking would be required.  

By Denis Dube, Summer 2005
"""

from DrawConnections import simpleConnection

class pacController:
  """
  Control logic for the *cough* PacMan "game"
  """

  def __init__(self, controller):
    self.controller = controller
    self.atom3i = controller.parent
    self.pacMan = None

    self.bindings()
  
  def bindings(self):
    """ Keybinds work with the numpad if it's in numeric mode... """
  
    b = self.atom3i.parent.bind

    b("<KeyPress-4>", lambda event, s=self: s.left(event))
    b("<KeyPress-6>", self.right)
    b("<KeyPress-8>", self.up)
    b("<KeyPress-2>", self.down)
    
    print 'Bindings keys 2, 4, 6, 8 on keypad for moving PacMan'
    
      
  def getPacMan(self):
    """ Assumes only one PacMan on the board... """
    pacManNodes = self.atom3i.ASGroot.listNodes['pacmanV3']
    if(not pacManNodes): 
      return None
    else:
      return pacManNodes[0]
    
  def getScoreboard(self):
    """ Assumes only one Scoreboard on the board... """
    scoreBoards = self.atom3i.ASGroot.listNodes['ScoreBoard']
    if(not scoreBoards): 
      return None
    else:
      return scoreBoards[0]
    
    
  def eat(self, gridNode):
    """ Eat food if the node has some... """
    foodLink = None
    for conn in gridNode.in_connections_:
      if(conn.__class__.__name__ == 'foodLinkV3'):
        foodLink = conn
        break
    if(not foodLink): return
    food = foodLink.in_connections_[0]
        
    # Eat the food!
    self.atom3i.deleteRealEntity(food.graphObject_.tag, food.graphObject_)
    print 'Mmmmm, PacMan ate the bait!'
    self.updateScore()
    
  
  def updateScore(self):
    """ Each time PacMan eats, the score goes up! """
    scoreBoard = self.getScoreboard()
    if(not scoreBoard): return
    scoreVal = scoreBoard.score.getValue()
    scoreBoard.score.setValue(scoreVal+1)
    scoreBoard.graphObject_.ModifyAttribute('score',scoreVal+1)
      
  def left(self, event=None):
    """ Make PacMan move left if possible """
    print 'go left pacman'
    
    # Get the PacMan semantic node
    pacMan = self.getPacMan()
    if(not pacMan): return
    
    # Get the grid node that PacMan is sitting on, and it's coords
    if(not pacMan.out_connections_): return
    gridLink = pacMan.out_connections_[0]
    gridNode = gridLink.out_connections_[0]
    (x, y) = (gridNode.graphObject_.x, gridNode.graphObject_.y)

    # Search over all the nodes for one with coords left of this node
    # NOTE: this is okay since PacMan boards tend to be very small...
    closestMatch = None
    gridNodes = self.atom3i.ASGroot.listNodes['gridNodeCenter']
    for node in gridNodes:
      if(node.graphObject_.x < x 
         and node.graphObject_.x > x - gridNode.graphObject_.qcW.get() * 1.2
         and node.graphObject_.y == y):
        closestMatch = node
        break
    if(not closestMatch):
      print 'Can not move left!'
      return
        
    # Delete the link between PacMan and the old grid node
    self.atom3i.deleteRealEntity(gridLink.graphObject_.tag, gridLink.graphObject_)
    # Create a link between PacMan and the new grid node
    simpleConnection(self.atom3i, pacMan, closestMatch)
    # Make sure PacMan can be seen when the constraints move him...
    pacMan.graphObject_.raiseItem()      
    # Eat food on the new node if there's some...
    self.eat(closestMatch)
    
  def right(self, event=None):    
    print 'go right pacman'
    
    pacMan = self.getPacMan()
    if(not pacMan): return
        
    if(not pacMan.out_connections_): return
    gridLink = pacMan.out_connections_[0]
    gridNode = gridLink.out_connections_[0]
    (x, y) = (gridNode.graphObject_.x, gridNode.graphObject_.y)

    closestMatch = None
    gridNodes = self.atom3i.ASGroot.listNodes['gridNodeCenter']
    for node in gridNodes:
      if(node.graphObject_.x > x 
         and node.graphObject_.x < x + gridNode.graphObject_.qcW.get() * 1.2
         and node.graphObject_.y == y):
        closestMatch = node
        break
    if(not closestMatch):
      print 'Can not move right!'
      return
        
    self.atom3i.deleteRealEntity(gridLink.graphObject_.tag, gridLink.graphObject_)
    simpleConnection(self.atom3i, pacMan, closestMatch)
    
    pacMan.graphObject_.raiseItem()  
    self.eat(closestMatch)
    
  def up(self, event=None):
    print 'go up pacman'
    
    pacMan = self.getPacMan()
    if(not pacMan): return
    
    if(not pacMan.out_connections_): return
    gridLink = pacMan.out_connections_[0]
    gridNode = gridLink.out_connections_[0]
    (x, y) = (gridNode.graphObject_.x, gridNode.graphObject_.y)

    closestMatch = None
    gridNodes = self.atom3i.ASGroot.listNodes['gridNodeCenter']
    for node in gridNodes:
      if(node.graphObject_.y < y 
         and node.graphObject_.y > y - gridNode.graphObject_.qcH.get() * 1.2
         and node.graphObject_.x == x):
        closestMatch = node
        break
    if(not closestMatch):
      print 'Can not move up!'
      return
                  
    self.atom3i.deleteRealEntity(gridLink.graphObject_.tag, gridLink.graphObject_)
    simpleConnection(self.atom3i, pacMan, closestMatch)
    
    pacMan.graphObject_.raiseItem()  
    self.eat(closestMatch)    
    
  def down(self, event=None):
    print 'go down pacman'
    
    pacMan = self.getPacMan()
    if(not pacMan): return
    
    if(not pacMan.out_connections_): return
    gridLink = pacMan.out_connections_[0]
    gridNode = gridLink.out_connections_[0]
    (x, y) = (gridNode.graphObject_.x, gridNode.graphObject_.y)

    closestMatch = None
    gridNodes = self.atom3i.ASGroot.listNodes['gridNodeCenter']
    for node in gridNodes:
      if(node.graphObject_.y > y 
         and node.graphObject_.y < y + gridNode.graphObject_.qcH.get() * 1.2
         and node.graphObject_.x == x):
        closestMatch = node
        break
    if(not closestMatch):
      print 'Can not move down!'
      return
        
    self.atom3i.deleteRealEntity(gridLink.graphObject_.tag, gridLink.graphObject_)
    simpleConnection(self.atom3i, pacMan, closestMatch)
    
    pacMan.graphObject_.raiseItem()  
    self.eat(closestMatch)
    