from ATOM3TypeDialog import *
from ATOM3Text import *
import os
import os.path

BASEDIR = os.path.split( __file__ )[ 0 ] 

class ConservationAnalysis:

  def __init__(self):
    # global variables
    self.places = []
    self.index = []
    self.gamma = []
    self.capacity = []
    self.lastNonZeroRow = 0
    self.count = 0
    self.cStr = ''
    self.multStr = ''
    self.markingCol = []
    self.permPlaces = []
    self.indepIndex = []

  ##########################################################
  # check is the method called by the coverability tree    #
  # in Atom3, which will analyse the conservation property #
  # of the petrinet                                        #
  ##########################################################
  def check(self, parentApp, model):
    
    # ==================================================
    # First we have to parse the info that we got from   
    # the coverability tree to get the places name and   
    # the number of token in each place for each marking 
    # ==================================================

    marking = self.parseTree(parentApp, model)
    mark = self.clone(marking)
    self.permPlaces = self.vecClone(self.places)

    times = 0
    num = self.permNum(len(self.places),1)

    while times < num:
      # =================================================
      # Once we have all the marking in a matrix form, we  
      # need to reduce it using Gauss-Jordan algorithm     
      # =================================================

      reducedMarking = self.reduce(mark)

      # check if we have all 1's in all first non-zero column
      allOne = 1
      i = 0
      while i < len(reducedMarking):
        j = 0
	while j < len(reducedMarking[i]):
	  if reducedMarking[i][j] != 0.0:
	    tmpVal = reducedMarking[i][j]
	    if tmpVal < 0:
	      tmpVal = tmpVal * -1
	    if tmpVal != 1.0:
	      allOne = 0
	    break
	  j = j+1
	if allOne == 0:
	  break
	i = i+1

      # if we don't have all 1's in all first non-zero column
      # do permutation
      if allOne == 0:
        permOrder = self.getPermuteOrder(len(self.places),times)
	mark = self.permute(permOrder)
	print "permutation: ", times
	print ""

      else:
      
        # ===================================================
        # Once we have the reduce form of the marking matrix,
        # we can find the values for the gammas
        # ===================================================

        self.solve(reducedMarking)

#      print "solution: "
#      i = 0
#      while i < len(self.gamma):
#        print self.gamma[i]
#	i = i+1
#      print ""
#      break

      # check if we have enough conservation equations
        if len(self.gamma) == len(self.indepIndex):
          break
        else:
          # permute and start over again
	  permOrder = self.getPermuteOrder(len(self.places),times)
	  mark = self.permute(permOrder)

	  print "permutation: ", times
          print ""
	  
      times = times+1


    # put conservation rules in array
    result = []
    i = 0
    while i < len(self.gamma):
      j = 0
      first = 1
      tmpStr = ''
      while j < len(self.gamma[i]):
	if self.gamma[i][j] != 0:
	  if first == 1:
	    tmpStr += str(self.gamma[i][j]) + ' x[' + self.permPlaces[j] + ']'
	    first = 0
	  else:
	    tmpStr += ' + ' + str(self.gamma[i][j]) + ' x[' + self.permPlaces[j] + ']'
	j = j+1
      tmpStr += ' = ' + str(self.capacity[i])
      result.append(tmpStr)
      i = i+1



    # ===============================================
    # Finally, we display the results on a new window
    # ===============================================
    
    res_data = '====================================\n'
    res_data += '=                                  =\n'
    res_data += '=  Conservation Analysis Results:  =\n'
    res_data += '=                                  =\n'
    res_data += '====================================\n\n\n'
    

    i = 0
    while i < len(result):
      res_data += '\t' + result[i] + '\n\n'
      i = i+1

    resText = ATOM3Text(res_data, 110, 30)
    at3d = ATOM3TypeDialog(model.parent, resText)

  #######################################################
  # parseTree get the info from the coverability tree,  #
  # retrieves all the places name and puts all markings #
  # in a matrix form                                    #
  #                                                     #
  # it returns a the matrix of marking                  #
  #######################################################
  def parseTree(self, parentApp, model):
    self.places = []
    mark = []

    # ========================================================
    # Iterate through all RGnode nodes to retrieve places name
    #
    # For all the marking, get all the places name.
    # If we don't already have it, add it.
    # ========================================================

    for node in model.listNodes["RGnode"]:
      # store the string and parse it
      str = node.state.getValue()

      tmpList = []
      openBracket = 0
      comma = 0
      word = ""
      
      # ==============================================
      # Loop over all character in str
      # For each place, add it to the tmpList
      #
      # Format of each str: [p1, p3(2)] or [p1(2), p2]
      # ==============================================
     
      i = 0
      while i < len(str):
        if str[i] == "(":
          openBracket = 1
        elif str[i] == ")":
          openBracket = 0
        elif str[i] == "]":
	  tmpList.append(word)
	  word = ""
        elif str[i] == ",":
          comma = 1
	  tmpList.append(word)
	  word = ""
        elif str[i] == " ":
          if comma == 1:
	    comma = 0
          else:
	    word = word + str[i]
        elif str[i] != "[":
          if openBracket == 0:
	    word = word + str[i]
        i = i+1

      # ========================================
      # Add the new places to our list of places
      # ========================================

      for element in tmpList:
        i = 0
        if len(self.places) == 0:
          self.places.append(element)
        else:
          for place in self.places:
            if element != place:
	      i = i+1

          # place is not found in self.places yet
          if i == len(self.places):
            self.places.append(element)

#    print "\nplaces: "
#    print self.places
#    print ""
      
    
    # ================================================
    # Iterate through all RGnode nodes to retrieve all
    # markings and put them in a matrix form
    # ================================================

    for node in model.listNodes["RGnode"]:
      str = node.state.getValue()

      tmpList = []
      openBracket = 0
      word = ""

      # =================================================
      # Initialize all the elements in tmpList to 0.0
      # since the marking received from the coverability
      # tree only contain those that actually have tokens
      # in them. Therefore, all the rest are 0.0
      # =================================================

      i = 0
      while i < len(self.places):
        tmpList.append(0.0)
	i = i+1

      # ======================================================
      # Now we read the marking received from the coverability
      # tree and update tmpList
      # ======================================================

      i = 0
      while i < len(str):
        if str[i] == "(":
          openBracket = 1
        elif str[i] == ")":
          openBracket = 0
        elif str[i] == "]":
	  indx = self.places.index(word)
	  if tmpList[indx] == 0:
	    tmpList[indx] = 1.0
	  word = ""
        elif str[i] == ",":
          comma = 1
	  indx = self.places.index(word)
	  if tmpList[indx] == 0:
	    tmpList[indx] = 1.0
	  word = ""
        elif str[i] == " ":
          if comma == 1:
	    comma = 0
          else:
	    word = word + str[i]
        elif str[i] != "[":
          if openBracket == 0:
	    word = word + str[i]
	  else:
	    indx = self.places.index(word)
	    tmpList[indx] = self.parseInt(str[i],tmpList[indx])
        i = i+1

      mark.append(tmpList)

#    print "Markup matrix:"
#    i = 0
#    while i < len(mark):
#      print mark[i]
#      i = i+1
#    print ""

    # break down the matrix into column
    j = 0
    while j < len(mark[0]):  # for each column
      tmpCol = []
      i = 0
      while i < len(mark):   # for each row
        tmpCol.append(mark[i][j])
	i = i+1
      self.markingCol.append(tmpCol)
      j = j+1

#    print "Markup in column:"
#    i = 0
#    while i < len(self.markingCol):
#      print self.markingCol[i]
#      i = i+1
#    print ""

    return mark

  ##############################################
  # reduce reduces the matrix of marking using #
  # the Gauss-Jordan elimination algorithm     #
  #                                            #
  # it returns a reduced matrix of marking     #
  ##############################################
  def reduce(self, matrix):
    numRow = len(matrix)

    # ==========================================
    # Augment the matrix with -C in first column
    # ==========================================

    i = 0
    while i < numRow:
      matrix[i].insert(0,-1.0)
      i = i+1
    
    numCol = len(matrix[0])
    self.lastNonZeroRow = 0
    stopReduce = 0

    # ============================
    # Step 1: Do forward reduction
    # ============================
    
    i = 0
    while i < numRow:  
      if i == numCol:
        break

      # =========================================
      # We want to find a pivot to guarantee that
      # we won't divide by zero
      #
      # If a column has all zero entries, move to
      # the next column and find its pivot
      # =========================================

      max = 0
      c = i
      while max == 0:
        if c == numCol:
	  self.lastNonZeroRow = i-1
	  stopReduce = 1
	  break

        # Find a pivot (max value of the column)
	# Here we are only dealing with absolute values
	j = i+1
	if matrix[i][c] < 0:
	  max = matrix[i][c] * -1
	else:
	  max = matrix[i][c]
	
	row = i
	while j < numRow:
	  tmp = matrix[j][c]
	  if tmp < 0:
	    tmp = tmp * -1
	  if tmp > max:
	    max = tmp
	    row = j
	  j = j+1

	# If we found a pivot, which is not on row i, swap
	if row != i:
	  tmp = matrix[i]
	  matrix[i] = matrix[row]
	  matrix[row] = tmp

	if max == 0:
	  c = c+1

      # stop reducing?
      if stopReduce == 1:
	break

      # ========================
      # Forward reduce algorithm
      # ========================

      j = i+1
      while j < numRow:
        k = len(matrix[i])-1
	while k >= c:
	  matrix[j][k] -= matrix[i][k]*matrix[j][c]/matrix[i][c]
	  k = k-1
	j = j+1
      
      i = i+1

    # =============================
    # Step 2: Do backward reduction
    # =============================

    if self.lastNonZeroRow == 0:
      self.lastNonZeroRow = numRow - 1

    i = self.lastNonZeroRow
    while i >= 0:
      # get the first non-zero column on row i
      c = i
      col = i

      if matrix[i][c] == 0:
        while c < numCol:
	  if matrix[i][c] != 0:
	    col = c
	    break
	  c = c+1

      # =========================
      # Backward reduce algorithm
      # =========================
      
      j = i-1
      while j >= 0:
        k = len(matrix[i])-1
	while k >= col:
	  matrix[j][k] -= matrix[i][k]*matrix[j][col]/matrix[i][col]
	  k = k-1
	j = j-1

      i = i-1

    # ===============================================
    # Step 3: Each row should be in its simplest form
    #         ie. rows of 2s should become rows of 1s
    # ===============================================

    i = 0
    while i <= self.lastNonZeroRow:
      j = 0
      data = []
      while j < numCol:
        # only deal with absolute value
	tmp = matrix[i][j]
	if tmp < 0:
	  tmp = tmp * -1
	if tmp != 0:
	  data.append(tmp)
	j = j+1

      # ==================================================
      # get largest common divisor for all entries in data
      # ==================================================

      # get the max of the entries
      maxDivisor = self.getLargestDivisor(data)
      
      # divide the entries in the matrix
      if maxDivisor > 1:
        j = 0
	while j < len(matrix[i]):
	  matrix[i][j] = matrix[i][j]/maxDivisor
	  j = j+1
	
      i = i+1

#    print "final matrix: "
#    i = 0
#    while i < numRow:
#      print matrix[i]
#      i = i+1
#    print ""

    return matrix


  ############################################
  # solve setup a linear programming problem #
  #                                          #
  #                                          #
  ############################################
  def solve(self, matrix):
    numCol = len(matrix[0])
  
    # ================================================
    # Step 1:
    #
    #    1. Count the number of independences
    #    2. Store the equation for C
    #    3. Store the other equations in a matrix form
    # ================================================

    self.C = []
    self.mult = []

    # traverse the matrix
    i = 0
    col = 1
    while i <= self.lastNonZeroRow:
      # the first line is C
      if i == 0:
        j = 1
	neg = 0
	if matrix[i][j-1] < 0:
	  neg = 1

	while j < numCol:
	  if matrix[i][j] != 0:
	    if neg == 0:
	      self.C.append(matrix[i][j] * -1)
	    else:
	      self.C.append(matrix[i][j])
	  else:
	    self.C.append(matrix[i][j])
	  j = j+1
      else:
        while col < numCol:
	  # check which variables are the dependent ones
	  if matrix[i][col] == 0:
	    self.mult.append(-1)
	    col = col+1
	  else:
	    j = col+1
	    neg = 0
	    tmpList = []
	    if matrix[i][col] < 0:
	      neg = 1
	      tmpList.append(matrix[i][col] * -1)
	    else:
	      tmpList.append(matrix[i][col])

	    # store all variable the dependent one depends on
	    while j < numCol:
	      if matrix[i][j] != 0:
	        if neg == 0:
		  tmpList.append(matrix[i][j] * -1)
		else:
		  tmpList.append(matrix[i][j])
              else:
	        tmpList.append(matrix[i][j])
	      j = j+1

	    self.mult.append(tmpList)
	    col = col+1
	    break
      i = i+1

#    print "C: ", self.C
#    print ""
#    i = 0
#    print "mult: "
#    while i < len(self.mult):
#      print self.mult[i]
#      i = i+1
#    print ""
	
    
    # ========================================
    # Step 2:
    #
    #    Set up the linear programming problem
    # ========================================

    # store index of non-zero value entries in C
    self.index = []
    i = 0
    while i < len(self.C):
      if self.C[i] != 0:
	self.index.append(i)
      i = i+1

#    print "index: ", self.index
#    print ""

    # store the index of all independent variables
    self.indepIndex = self.vecClone(self.index)
    i = 0
    while i < len(self.mult):
      if self.mult[i] != -1:
        j = 1
	while j < len(self.mult[i]):
	  if self.mult[i][j] != 0:
	    tmp = j+i
	    # check if tmp is already in self.indepIndex
	    k = 0
	    while k < len(self.indepIndex):
	      if self.indepIndex[k] == tmp:
	        break
	      elif tmp < self.indepIndex[k]:
	        self.indepIndex.insert(k,tmp)
		break
	      k = k+1
	  j = j+1
      i = i+1


    # store strings to be written to files
    i = 0
    self.cStr = ''
    first = 1
    while i < len(self.C):  # for each entry of C
      if self.C[i] != 0:
	if first == 1:
          self.cStr += str(self.C[i]) + ' g' + str(i)
	  first = 0
	else:
	  self.cStr += ' + ' + str(self.C[i]) + ' g' + str(i)
      i = i+1

    self.multStr = ''
    i = 0
    while i < len(self.mult):
      if self.mult[i] != -1:
	j = 1
	first = 1
	while j < len(self.mult[i]):
	  if self.mult[i][j] != 0:
	    if first == 1:
	      self.multStr += str(self.mult[i][j]) + ' g' + str(i+j)
	      first = 0
	    else:
	      self.multStr += ' + ' + str(self.mult[i][j]) + ' g' + str(i+j)
	  j = j+1
	self.multStr += ' >= 0;\n'
      i = i+1

   
    # get the max values of coefficient in equation of C
    # C can only be as big as the max coef
    i = 0
    maxC = 0
    while i < len(self.C):  # for each entry of C
      if self.C[i] > maxC:
        maxC = self.C[i]
      i = i+1

#    print "max coef: ", maxC
#    print ""

    # write to file to solve linear programming problem
    self.gamma = []
    self.capacity = []
    self.count = 0
    num = len(self.index)
    capVal = 1

    while capVal <= maxC:
      i = -1
      while i < len(self.indepIndex):
        self.solveLinearProb(i,capVal)
	i = i+1
      # if not enough equation
      if self.count == len(self.indepIndex):
        break
      capVal = capVal+1
      

    # ==========================================
    # Put the result of the analysis in an array
    # ==========================================

    # traverse the gamma matrix
    i = 0
    while i < len(self.gamma):
      # traverse the mult matrix
      j = 0
      while j < len(self.mult):
	if self.mult[j] != -1:
	  tmpVal = 0.0
	  k = 1
	  while k < len(self.mult[j]):
	    tmpVal += self.mult[j][k]*self.gamma[i][j+k]
	    k = k+1
	  self.gamma[i][j] = tmpVal
	j = j+1
      i = i+1



  #######################################
  # solve linear problem using lp_solve #
  #                                     #
  # option: 0 - modify gamma values     #
  # option: 1 - modify C values         #
  #######################################
  def solveLinearProb(self, indx, CVal):
    fileStr = './lp' + str(indx)
    f = open(fileStr + '.lp', 'w')
    f.write('min: ' + self.cStr + ';\n\n')
    f.write(self.multStr + '\n')

    j = 0
    while j < len(self.indepIndex):
      if j == indx:
        f.write('g' + str(self.indepIndex[j]) + ' >= 1;\n')
      else:
	f.write('g' + str(self.indepIndex[j]) + ' >= 0;\n')
      j = j+1

    # write constraint on C
    f.write('\n' + self.cStr + ' >= ' + str(CVal) + ';\n')

    # write constraint that all variables are integers
    j = 0
    first = 1
    f.write('\nint ')
    while j < len(self.indepIndex):
      if first == 1:
	f.write('g' + str(self.indepIndex[j]))
	first = 0
      else:
	f.write(', g' + str(self.indepIndex[j]))
      j = j+1
    f.write(';')

    f.close()

    # =============================================
    # Now call lp_solve to solve the linear problem
    # =============================================

    solvercmd = "lp_solve_4.0/"
    os.system(os.path.join( BASEDIR, solvercmd )+"lp_solve < "+fileStr+".lp > "+fileStr+".out")

    # ====================================
    # Parse the file generated by lp_solve
    # ====================================

    # initialize tmpGamma to store gamma values for all vars
    tmpCap = 0.0
    tmpGamma = []
    j = 0
    while j < len(self.C):
      tmpGamma.append(0.0)
      j = j+1

    f = open(fileStr + '.out', 'r')

    # retrieve the C value
    f.readline()
    line = f.readline()
    j= 0
    while j < len(line):
      tmpCap = self.parseInt(line[j], tmpCap)
      j = j+1

    # retrieve the gamma values
    f.readline()
    f.readline()
    line = f.readline()
    while line != "":
      j = 0
      indx = 0
      value = 0
      gFound = 0
      while j < len(line):
	if line[j] == ' ':
          gFound = 1
	if gFound == 0:
	  indx = self.parseInt(line[j], indx)
	else:
	  value = self.parseInt(line[j], value)
	j = j+1
      
      tmpGamma[indx] = value + 0.0
      # get next line
      line = f.readline()

    # =============================================
    # Check if the new set of gamma found is unique
    # If yes, add it
    # =============================================

    # reduce the new set of gamma to simplest form
    maxDivisor = self.getLargestDivisor(tmpGamma)

    # divide the entries in the matrix
    if maxDivisor > 1:
      j = 0
      while j < len(tmpGamma):
	tmpGamma[j] = tmpGamma[j]/maxDivisor
        j = j+1

    # traverse through gamma
    j = 0
    duplicate = 0
    while j < len(self.gamma):
      k = 0
      countCol = 0
      while k < len(self.gamma[j]):
	if tmpGamma[k] == self.gamma[j][k]:
	  countCol = countCol+1
	k = k+1

      if countCol == len(self.gamma[j]):
	duplicate = 1
      j = j+1

    if duplicate == 0:
      self.gamma.append(tmpGamma)
      self.capacity.append(tmpCap)
      self.count += 1
	
	
  #########################################
  # convert string number to number value #
  #                                       #
  # it returns the updated value          #
  #########################################
  def parseInt(self, strVal, val):
    if strVal == '0': val = val*10
    elif strVal == '1': val = val*10 + 1
    elif strVal == '2': val = val*10 + 2
    elif strVal == '3': val = val*10 + 3
    elif strVal == '4': val = val*10 + 4
    elif strVal == '5': val = val*10 + 5
    elif strVal == '6': val = val*10 + 6
    elif strVal == '7': val = val*10 + 7
    elif strVal == '8': val = val*10 + 8
    elif strVal == '9': val = val*10 + 9

    return val


  ######################################
  # returns the largest common divisor #
  ######################################
  def getLargestDivisor(self, data): 
    # get the max of the entries
    j = 0
    maxEntry = 0
    while j < len(data):
      if data[j] > maxEntry:
        maxEntry = data[j]
      j = j+1

    # get largest common divisor
    maxDivisor = 1
    while maxEntry > 0:
      j = 0
      divCount = 0
      while j < len(data):
	if data[j]%maxEntry == 0:
	  divCount = divCount+1
	j = j+1
      if divCount == len(data):
	maxDivisor = maxEntry
	break
      maxEntry = maxEntry-1

    return maxDivisor


  ###############################
  # returns a clone of a matrix #
  ###############################
  def clone(self, matrix):
    dup = []
    i = 0
    while i < len(matrix):
      j = 0
      row = []
      while j < len(matrix[i]):
        row.append(matrix[i][j])
	j = j+1
      dup.append(row)
      i = i+1

    return dup

  ##############################
  # return a clone of a vector #
  ##############################
  def vecClone(self, vector):
    dup = []
    i = 0
    while i < len(vector):
      dup.append(vector[i])
      i = i+1
    return dup


  #####################################
  # returns the number of permutation #
  #####################################
  def permNum(self, length, stop):
    mult = 1
    step = length
    while step >= stop:
      mult = mult * step
      step = step-1
      
    return mult


  #################################
  # returns the permutation order #
  #################################
  def getPermuteOrder(self, length, num):
    swap = 0
    bar = 0
    foo = num
    x = 0
    array = []
    while x < length:
      array.append(x)
      x = x+1

    x = 2
    while x <= length:
      lf = self.permNum(length,x+1)
      bar = foo/lf
      if bar:
        foo = foo - bar*lf
	swap = array[length-x]
	array[length-x] = array[length-x+bar]
	array[length-x+bar] = swap
      x = x+1

    return array


  #########################################################
  # Permute the matrix according to the permutation order #
  #                                                       #
  # it returns the permutated matrix                      #
  #########################################################
  def permute(self, order):
    mark = []
    # initialize the matrix
    i = 0
    while i < len(self.markingCol[0]):  # num of rows
      tmp = []
      j = 0
      while j < len(self.markingCol):   # num of columns
        tmp.append(0.0)
	j = j+1
      mark.append(tmp)
      i = i+1
    
    j = 0
    while j < len(order):  # for each element in the order array
      indx = order[j]
      i = 0
      while i < len(self.markingCol[0]):  # for all the rows
        mark[i][j] = self.markingCol[indx][i]
	i = i+1
      self.permPlaces[j] = self.places[indx]
      j = j+1
      
    return mark

