
def GenConstantsForStates ( ASGroot, file ):
   """iterate on all states, generating "DATA xxx := constant"
   """   
   constant = 0
   for blob in ASGroot.listNodes['Blob']:
      file.write("DATA "+blob.Name.toString()+" := "+str(constant)+"\n")
      constant = constant+1

def GenYES_NO ( file ):
   """
      Generates "DATA YES := 100, NO := 0"
   """
   file.write("DATA YES := 100, NO := 0\n")

def GenInitialState ( ASGroot, file, indent ):
   """
      Generates the code for the initial states in each orthogonal component

      TO DO: CHECK IF WE DO NOT HAVE ORTHOGONAL COMPONENTS...
   """
   for initial in ASGroot.listNodes["has_Initial"]:
       orthoName = initial.in_connections_[0].Name.toString()
       iniState  = initial.out_connections_[0].out_connections_[0].out_connections_[0].Name.toString()
       file.write(indent+orthoName+" := "+iniState+"\n")

def getOrthoName ( blob ):
   """
      Return the orthogonal component name of this node or "no_ortho" if the node is not inside of an orthogonal component
   """
   for obj in blob.in_connections_:
       if obj.getClass() == "has_inside":
           return obj.in_connections_[0].Name.toString()
   return "no_ortho"

def GenHyperEdges ( ASGroot, file, indent ):
   """
      Generates OOCSMP code for the Hyperedges :
   """
   for blob in ASGroot.listNodes["Blob"]:       # for all states
       orthoName = getOrthoName ( blob )        # get the orthogonal component name
       blobName  = blob.Name.toString()         # get the name of the state
       for edge in blob.out_connections_:       # for each edge in state
           if edge.getClass() == "Hyperedge":   # it is an hyperedge, generate code
               edgeEvent = edge.Event.getValue()[0].toString()          # get the 1st event. TO DO: consider list of events
               edgeAction= edge.Broadcast_Event.getValue()        # get the 1st action. TO DO: consider list of actions
               if len(edgeAction) > 0:
                  actionName = edgeAction[0].toString()
               else:
                  actionName = None 
               # 1st.: Write the ACTIONS for the Transition
               if actionName:
                  file.write(indent+"FCNSW( "+orthoName+" - "+blobName+" + "+orthoName+"DONE, , ")
                  file.write("FCNSW("+ edgeEvent + ", "+actionName+", ," + actionName + "), )\n")
               # TO DO: CONSIDER GUARDS
               # 2nd.: Write the State Change originated by the Transition
               file.write(indent+"FCNSW( "+orthoName+" - "+blobName+" + "+orthoName+"DONE, , ")
               file.write("FCNSW("+ edgeEvent + ", "+ orthoName+" := "+blobName+ ", ," + orthoName+" := "+blobName + "), )\n")
               # 3rd.: Set DONE := YES
               file.write(indent+"FCNSW( "+orthoName+" - "+blobName+" + "+orthoName+"DONE, , ")
               file.write("FCNSW("+ edgeEvent + ", "+ orthoName+"DONE := YES, , " + orthoName+"DONE := YES), )\n")
               file.write("\n")

def GenDONES ( ASGroot, file, indent ):
   """
      Generates the "DONE<ortho> := NO" which goes at the beginning of the method
   """
   for ortho in ASGroot.listNodes["Orthogonal"]:
       file.write(indent+"DONE"+ortho.Name.toString()+" := NO\n")

def GenFullFile ( ASGroot ):
   """
      Generate a full file with the code generation, supposing the model is not inside a method
   """
   file = open( ASGroot.Name.toString()+"_OOCSMP.csm", "w+t")
   file.write("* This file has been generated automatically by AToM3\n")
   file.write("* Code generation routine for OOCSMP v0.1 by Juan de Lara, 2002\n")
   GenConstantsForStates(ASGroot, file)
   GenYES_NO(file)
   file.write("INITIAL\n")
   GenInitialState(ASGroot, file, "   ")
   file.write("\n\n")
   file.write("DYNAMIC\n")
   GenHyperEdges(ASGroot, file, "   ")
   file.write("\n\n")
   file.close()

def GenInFile (ASGroot, file ):
   """
      Generates code in a given file...
   """
   GenHyperEdges(ASGroot, file, "   ")
   file.write("\n\n")

def GenInFileData (ASGroot, file ):
   """
     Generates code (only for data)
   """
   GenConstantsForStates(ASGroot, file)
   GenYES_NO(file)

def GenInitialStates (ASGroot, file ):
   """
      Generates code for the INITIAL Block
   """
   GenInitialState(ASGroot, file, "   ")
