import ATOM3
import PQ
from random import randrange
from random import randint
from time import sleep

#STEP = 3.0

#########
#
# Metodo principal del simulador. Recibe una instancia de la clase ASG.
#
# Mejoras posibles:
#   - Aclarar el tema de acceso a conexiones y a tuplas de eventos (sustituir [] por funciones??)
#   - Control de avance de tiempo por el usuario
#   - Eleccion de transiciones: control por parte del usuario??
#   - t y eventsPQ globales??
#
#########
def TPPN_Simulate(atom3):
    print("\n---------->")
    print("+ Comienzo de la simulacion de la red de petri: ")

    # Cola de prioridad de eventos inicialmente vacia
    #  - Formato de eventos en cola: (timeWhenAvailable, PlaceID, numTokens)
    print("++ Inicializacion de cola de prioridad de eventos")
    eventsPQ = PQ.PriorityQeue()
  
    # Recorrido inicial de la informacion del modelo, metiendo en la cola de prioridad los tokens
    # no disponibles de cada estado
    print("++ Almacenando lista de places")    
    places = atom3.ASGroot.listNodes['TPPN_TimedPlace']
    placesIndex = {} ## Diccionario para acceso directo a elementos de places por su nombre
    print("++ Almacenando lista de transitions")    
    transitions = atom3.ASGroot.listNodes['TPPN_Transition']

    # Almacenamos el tiempo absoluto en t
    print("++ Tiempo = 0")
    t = 0
    
    print("++ Recorrido inicial de tokens no disponibles en el modelo e inicializacion de diccionario de nombres e indices")
    for place in places:
        placesIndex[place.name.getValue()] = places.index(place)
        if place.delayedTokens.getValue() > 0:
            print("+++ Introduciendo en la cola -> (" + str(t + place.timeAtPlace.getValue()) + ", " + str(place.name.getValue())+", "+ str(place.delayedTokens.getValue()) + ")")
            eventsPQ.insert( (t + place.timeAtPlace.getValue(), place.name.getValue(), place.delayedTokens.getValue()) )
            
    eventsPQ.dump()

    # Bucle principal de la simulacion
    print("++ Bucle principal de la simulacion")
    end = 0 # Flag para marcar fin de simulacion
    while end != 1:
        print("+++ Ejecucion de eventos que ocurren en t=" + str(t))
        while (not eventsPQ.isEmpty()) and (eventsPQ.peek()[0] <= t) :
            event = eventsPQ.extract()
            print("++++ Ejecutando evento ("+str(event[0])+", "+str(event[1])+", "+str(event[2])+")")
            # Pasamos tokens de delayed a available y actualizamos el modelo
            activatePlaceTokens(places[placesIndex[event[1]]], event[2])
        print("+++ Fin de ejecucion de eventos")

        # Debug: estado de la cola               
        eventsPQ.dump()
                    
        # TO DO: Comprobar activacion de transiciones y ejecutarlas con sus consecuencias
        # OJO: la ejecucion de una transicion puede acarrear la activacion de otras
        print("+++ Disparo de transiciones")        
        stateChanged = 1
        while stateChanged == 1 :
            print("++++ Comprobacion de activacion de transiciones")
            activeTransitions = []
            for transition in transitions :
                print("+++++ Comprobando activacion de transicion: "+ str(transition.name.getValue()))
                active = 1
                # Si la transicion no tiene arcos de entrada, esta activada
                if len(transition.in_connections_) != 0 :
                    for arc in transition.in_connections_ :
                        # Solo debe haber una place por arco P2T
                        place = arc.in_connections_[0]
                        print("++++++ Tokens en "+ str(place.name.getValue())+": "+str(place.availableTokens.getValue())+". Peso del arco: "+ str(arc.weight.getValue()))
                        if arc.weight.getValue() > place.availableTokens.getValue() :
                            active = 0
                            break

                if active == 1 :
                    print("+++++ Transicion "+ str(transition.name.getValue())+" activada")
                    activeTransitions.append(transition)

            print ("activas = "+ str(len(activeTransitions)))
            if len(activeTransitions)>0:
                # Seleccion aleatoria entre las transiciones activadas
                if len(activeTransitions) == 1:
                    transition = activeTransitions[0]                    
                else :
                    print("Aleat entre 0 y " + str(len(activeTransitions)-1))
                    transition = activeTransitions[randrange(0, len(activeTransitions))] # randrange no incluye el ultimo elemento
                    #transition = activeTransitions[randint(0,len(activeTransitions)-1)]
                print("++++ Disparando una transicion entre las activadas")
                fireTransition(transition, t, eventsPQ)                
            else:                
                stateChanged = 0                
        print("+++ Fin de disparo de transiciones")
            
        
        print("+++ Calculo de siguiente tiempo relevante (minimo de la cola)")
        if eventsPQ.isEmpty() :
            print("+++ No quedan eventos en la cola ni transiciones a ejecutar")
            end = 1
        else :
            t = eventsPQ.peek()[0]
            
    print("++ Fin de bucle principal")    
    print("+ Fin de la simulacion")
    print("<----------\n")

# Activacion de cierto numero de tokens de un place
def activatePlaceTokens(place, numTokens):
    print("----> Pasando tokens de delayed a available")

    #TO DO:
    #place.graphObject_.HighLight(1)

    print("** Previo estado de place -> Place: "+place.name.getValue()+", Delayed: "+str(place.delayedTokens.getValue())+", Available:"+str(place.availableTokens.getValue()))
    place.delayedTokens.setValue(place.delayedTokens.getValue() - numTokens)
    place.availableTokens.setValue(place.availableTokens.getValue() + numTokens)
    print("** Nuevo estado de place -> Place: "+place.name.getValue()+", Delayed: "+str(place.delayedTokens.getValue())+", Available:"+str(place.availableTokens.getValue()))
    # Actualizamos la representacion grafica de los nodos
    place.graphObject_.ModifyAttribute("delayedTokens", place.delayedTokens.toString())
    place.graphObject_.ModifyAttribute("availableTokens", place.availableTokens.toString())

    #TO DO:
    #sleep(STEP)
    #place.graphObject_.HighLight(0)

    print("----> Tokens pasados de delayed a available")

# Ejecucion de transicion, mete en la cola de eventos los nuevos delayed tokens generados
def fireTransition(transition, t, eventsPQ):
    print("----> Ejecutando transicion " + str(transition.name.getValue()))
    
    #TO DO:
    #transition.graphObject_.HighLight(1)

    print("** Restando tokens en places origen")
    for arc in transition.in_connections_ :
        # Solo debe haber una place por arco P2T
        place = arc.in_connections_[0]
        print("*** Peso del arco: "+ str(arc.weight.getValue()))
        print("*** Tokens iniciales en "+ str(place.name.getValue())+": "+str(place.availableTokens.getValue()))
        place.availableTokens.setValue(place.availableTokens.getValue() - arc.weight.getValue())
        print("*** Tokens finales en "+ str(place.name.getValue())+": "+str(place.availableTokens.getValue()))
        # Actualizacion de objeto grafico
        place.graphObject_.ModifyAttribute("availableTokens", place.availableTokens.toString())

    print("** Sumando tokens en places destino (available o delayed segun TTA)")
    for arc in transition.out_connections_ :
        # Solo debe haber una place por arco T2P
        place = arc.out_connections_[0]
        print("*** Peso del arco: "+ str(arc.weight.getValue()))
        # Si timeAtPlace es = 0 incrementamos el contador de availables
        if place.timeAtPlace.getValue() == 0 :
            print("*** Available tokens iniciales en "+ str(place.name.getValue())+": "+str(place.availableTokens.getValue()))
            place.availableTokens.setValue(place.availableTokens.getValue() + arc.weight.getValue())
            print("*** Available tokens finales en "+ str(place.name.getValue())+": "+str(place.availableTokens.getValue()))
            # Actualizacion de objeto grafico
            place.graphObject_.ModifyAttribute("availableTokens", place.availableTokens.toString())

        # Si no, incrementamos el contador de delayed e introducimos eventos en la cola
        else:
            print("*** Delayed tokens iniciales en "+ str(place.name.getValue())+": "+str(place.delayedTokens.getValue()))
            place.delayedTokens.setValue(place.delayedTokens.getValue() + arc.weight.getValue())
            print("*** Delayed tokens finales en "+ str(place.name.getValue())+": "+str(place.delayedTokens.getValue()))
            # Actualizacion de objeto grafico
            place.graphObject_.ModifyAttribute("delayedTokens", place.delayedTokens.toString())
            # Introduccion del nuevo evento de tokens en la cola
            eventsPQ.insert( (t + place.timeAtPlace.getValue(), place.name.getValue(), arc.weight.getValue()) )

    #TO DO:
    #sleep(STEP)
    #transition.graphObject_.HighLight(0)

    print("<---- Fin de ejecucion de transicion " + str(transition.name.getValue()))
