package qoca;
// $Id: QcDelLinEqTableau.java.cpp,v 1.3 2000/11/21 04:26:13 pmoulder Exp $

//============================================================================//
// Written by Sitt Sen Chok                                                   //
//----------------------------------------------------------------------------//
// The QOCA implementation is free software, but it is Copyright (C)          //
// 1994-1999 Monash University.  It is distributed under the terms of the GNU //
// General Public License.  See the file COPYING for copying permission.      //
//                                                                            //
// The QOCA toolkit and runtime are distributed under the terms of the GNU    //
// Library General Public License.  See the file COPYING.LIB for copying      //
// permissions for those files.                                               //
//                                                                            //
// If those licencing arrangements are not satisfactory, please contact us!   //
// We are willing to offer alternative arrangements, if the need should arise.//
//                                                                            //
// THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY EXPRESSED OR  //
// IMPLIED.  ANY USE IS AT YOUR OWN RISK.                                     //
//                                                                            //
// Permission is hereby granted to use or copy this program for any purpose,  //
// provided the above notices are retained on all copies.  Permission to      //
// modify the code and to distribute modified code is granted, provided the   //
// above notices are retained, and a notice that the code was modified is     //
// included with the above copyright notice.                                  //
//============================================================================//

import java.util.*;
import java.lang.*;
import java.io.*;

class QcDelLinEqTableau extends QcLinEqTableau
{
	//-----------------------------------------------------------------------//
	// Constructors.                                                         //
	//-----------------------------------------------------------------------//
	public QcDelLinEqTableau(int hintRows, int hintCols, QcBiMapNotifier n)
	{
		super(hintRows, hintCols, new QcDelCoreTableau(hintRows, hintCols,
			new QcLinEqRowColStateVector()), n);
	}

	public QcDelLinEqTableau(int hintRows, int hintCols, QcCoreTableau tab,
			QcBiMapNotifier n)
		{ super(hintRows, hintCols, tab, n); }

	public QcDelLinEqTableau(DataInputStream s, QcBiMapNotifier n)
	{
		super(n);
		loadFromBinary(s);
	}

	public QcDelLinEqTableau(StringTokenizer token, QcBiMapNotifier n)
	{
		super(n);
		loadFromString(token);
	}

	protected QcDelLinEqTableau(QcBiMapNotifier n)
		{ super(n); }

	//-----------------------------------------------------------------------//
	// Query functions.                                                      //
	//-----------------------------------------------------------------------//
	public double getRHSVirtual(int ci)
		{ return ((QcDelCoreTableau)fCoreTableau).getRHSVirtual(ci); }

	public double getValueVirtual(int ci, int vi)
		// This is the similar to getValue except that it will always obtain
		// value from computation of the quasi-inverse and orignial constraint
		// coefficients.  Whereas getValue obtain value directly from the
		// solved form if the real tableau is used.
	{
		// Short cut evaluations
		switch (getRowCondition(ci)) {
			case QcLinEqRowState.fRedundant:
				return 0.0f;
			case QcLinEqRowState.fNormalised:
				if (isBasic(vi))
					return 0.0f;
				break;
			case QcLinEqRowState.fRegular:
				if (isBasic(vi)) {
					if (vi == getBasicVar(ci))
						return 1.0f;
					else
						return 0.0f;
				}
				break;
			case QcLinEqRowState.fInvalid:
				break;
		}

		return ((QcDelCoreTableau)fCoreTableau).getValueVirtual(ci,vi);
	}

	public boolean isFree(int vi)
		// Indicates if a particular variable is in use (i.e. has a
		// non-zero coefficient in any solved form (or original) constraint.
		{ return ((QcDelCoreTableau)fCoreTableau).isFree(vi); }

	public boolean isValidSolvedForm()
		// For debugging purposes to ensure that the real solved form is
		// consistant with the virtual solved form.
		{ return ((QcDelCoreTableau)fCoreTableau).isValidSolvedForm(); }

	//-----------------------------------------------------------------------//
	// Manipulation functions.                                               //
	//-----------------------------------------------------------------------//

#if 0 /* Not used. */
	public void decreaseRows(QcBiMapNotifier note)
		// Get rid of the last (deleted) row
		{ ((QcDelCoreTableau)fCoreTableau).decreaseRows(note); }
#endif

	public void decreaseColumns(QcBiMapNotifier note)
		// Get rid of the last (empty) column
		{ ((QcDelCoreTableau)fCoreTableau).decreaseColumns(note); }

	public int removeEq(int ci)
		// RemoveEq removes an original constraint with index ci from
		// the solved form without disturbing the solution.  This requires
		// removal of one of the solved form constraints.  The index of
		// the removed solved form constraint is returned or InvalidCIndex
		// if there is a serious error (e.g. ci invalid).  For the current
		// implementation: after a row is removed the CIndex values for
		// particular rows are unchanged.
		// The notifier is told to drop the constraint index.
	{
		double max = 0.0f;		// Maximal absolute value of coeff
		int maxIndex = QcTableau.fInvalidConstraintIndex;
		boolean foundRedund = false;
		int pivotRow;
		int nrows = getRows();

		if (ci < 0 || ci >= nrows || fCoreTableau.getARowDeleted(ci))
			return QcTableau.fInvalidConstraintIndex;
		else if (fCoreTableau.getMRowIndex(ci) != QcTableau.fInvalidConstraintIndex)
			return QcTableau.fInvalidConstraintIndex;

		// First look for a suitable redundant constraint, meanwhile
		// keep track of the maximal size quasiColCoeff.
		QcSparseMatrixColIterator quasiColCoeffs =
			new QcSparseMatrixColIterator(
				((QcDelCoreTableau)fCoreTableau).fM, ci);

		while (!quasiColCoeffs.atEnd()) { // N.B. This loop contains a break
			if (isRedundant(quasiColCoeffs.fIndex)) {
				foundRedund = true;
				break;
			} else {
				// Rather than use fabs(quasiColCoeffs.Value()) we use
				// arithmetic and take advantage of the optimiser.
				if (quasiColCoeffs.fValue > max) {
					max = quasiColCoeffs.fValue;
					maxIndex = quasiColCoeffs.fIndex;
				} else if (-quasiColCoeffs.fValue > max) {
					max = -quasiColCoeffs.fValue;
					maxIndex = quasiColCoeffs.fIndex;
				}

				// Note that the ratio used in IPL 55 (1995) page 112 is
				// directly related to the inverse of max since the RHS
				// value is constant and we take the absolute value of max.
			}

			quasiColCoeffs.increment();
		}

		// Best to abort rather than use an invalid index
		if (!foundRedund && maxIndex == QcTableau.fInvalidConstraintIndex)
			return QcTableau.fInvalidConstraintIndex;

		// delete selected row from fM, column ci from fM, row ci from fA
		if (foundRedund) {
			pivotRow = quasiColCoeffs.fIndex;
			setRowCondition(pivotRow, QcLinEqRowState.fNormalised);
		} else
			pivotRow = maxIndex;

		((QcDelCoreTableau)fCoreTableau).pivotQuasiInverse(pivotRow, ci);

		if (!foundRedund)
			unsolve(pivotRow);	// removes pivotRow from the basis

		// The situation now simulates that of a newly added row after
		// Eliminate has been called, but before a Pivot.
		setMRowIndex(ci, pivotRow);
		setARowIndex(pivotRow, ci);
		deleteRow(ci);
		fNotifier.dropConstraint(ci);

		return pivotRow;
	}

	public boolean removeVar(int vi)
		// Removes a free variable from the tableau, returns false if not free.
	{
		if (!isFree(vi))
			return false;

		int lastcol = getColumns() - 1;

		if (vi != lastcol) {	// Swap column vi with last column
			fCoreTableau.swapColumns(vi, lastcol);
			fRowColState.swapColumns(vi, lastcol);
			fNotifier.swapVariables(vi, lastcol);
		}

		decreaseColumns(fNotifier);
		return true;
	}

	//-----------------------------------------------------------------------//
	// Utility function.                                                     //
	//-----------------------------------------------------------------------//
	public void loadFromBinary(DataInputStream s)
	{
		QcLinEqRowColStateVector rsv = new QcLinEqRowColStateVector(s, false);
		QcDelCoreTableau tab = new QcDelCoreTableau(s, rsv);
		super.loadFromBinary(s, tab, rsv);
	}

	public void loadFromString(StringTokenizer token)
	{
		QcLinEqRowColStateVector rsv =
          		new QcLinEqRowColStateVector(token, false);
		QcDelCoreTableau tab = new QcDelCoreTableau(token, rsv);
		super.loadFromString(token, tab, rsv);
	}

	public void print()
	{
		super.print();
		fCoreTableau.print();
		System.out.println();
		System.out.println("Consistant: " + (isValidSolvedForm() ? "yes" : "no"));
		System.out.println();
	}

	public void printSuper()
		{ super.print(); }

	public void transitiveClosure(QcIntVector vars)
		{ ((QcDelCoreTableau)fCoreTableau).transitiveClosure(vars); }
}

/*
  Local Variables:
  mode:java
  c-file-style:"java8"
  fill-column:80
  End:
  vim: set filetype=java :
*/
